@@ -148,8 +148,8 @@ pub trait FieldMethods {
     /// If this is a bitfield, how many bits does it need?
     fn bitfield_width(&self) -> Option<u32>;
 
-    /// Is this field marked as `mutable`?
-    fn is_mutable(&self) -> bool;
+    /// Is this feild declared public?
+    fn is_public(&self) -> bool;
 
     /// Get the annotations for this field.
     fn annotations(&self) -> &Annotations;
@@ -415,8 +415,8 @@ impl FieldMethods for Bitfield {
         self.data.bitfield_width()
     }
 
-    fn is_mutable(&self) -> bool {
-        self.data.is_mutable()
+    fn is_public(&self) -> bool {
+        self.data.is_public()
     }
 
     fn annotations(&self) -> &Annotations {
@@ -443,7 +443,7 @@ impl RawField {
         comment: Option<String>,
         annotations: Option<Annotations>,
         bitfield_width: Option<u32>,
-        mutable: bool,
+        public: bool,
         offset: Option<usize>,
     ) -> RawField {
         RawField(FieldData {
@@ -452,7 +452,7 @@ impl RawField {
             comment,
             annotations: annotations.unwrap_or_default(),
             bitfield_width,
-            mutable,
+            public,
             offset,
         })
     }
@@ -475,8 +475,8 @@ impl FieldMethods for RawField {
         self.0.bitfield_width()
     }
 
-    fn is_mutable(&self) -> bool {
-        self.0.is_mutable()
+    fn is_public(&self) -> bool {
+        self.0.is_public()
     }
 
     fn annotations(&self) -> &Annotations {
@@ -588,7 +588,7 @@ where
         } else {
             bytes_from_bits_pow2(unit_align_in_bits)
         };
-        let size = align_to(unit_size_in_bits, align * 8) / 8;
+        let size = align_to(unit_size_in_bits, 8) / 8;
         let layout = Layout::new(size, align);
         fields.extend(Some(Field::Bitfields(BitfieldUnit {
             nth: *bitfield_unit_count,
@@ -891,8 +891,8 @@ pub struct FieldData {
     /// If this field is a bitfield, and how many bits does it contain if it is.
     bitfield_width: Option<u32>,
 
-    /// If the C++ field is marked as `mutable`
-    mutable: bool,
+    /// If the C++ field is declared `public`
+    public: bool,
 
     /// The offset of the field (in bits)
     offset: Option<usize>,
@@ -915,8 +915,8 @@ impl FieldMethods for FieldData {
         self.bitfield_width
     }
 
-    fn is_mutable(&self) -> bool {
-        self.mutable
+    fn is_public(&self) -> bool {
+        self.public
     }
 
     fn annotations(&self) -> &Annotations {
@@ -954,6 +954,8 @@ pub struct Base {
     pub kind: BaseKind,
     /// Name of the field in which this base should be stored.
     pub field_name: String,
+    /// Whether this base is inherited from publically.
+    pub is_pub: bool,
 }
 
 impl Base {
@@ -981,6 +983,11 @@ impl Base {
 
         true
     }
+
+    /// Whether this base is inherited from publically.
+    pub fn is_public(&self) -> bool {
+        self.is_pub
+    }
 }
 
 /// A compound type.
@@ -1106,21 +1113,17 @@ impl CompInfo {
         }
 
         // empty union case
-        if self.fields().is_empty() {
+        if !self.has_fields() {
             return None;
         }
 
         let mut max_size = 0;
         // Don't allow align(0)
         let mut max_align = 1;
-        for field in self.fields() {
-            let field_layout = field.layout(ctx);
-
-            if let Some(layout) = field_layout {
-                max_size = cmp::max(max_size, layout.size);
-                max_align = cmp::max(max_align, layout.align);
-            }
-        }
+        self.each_known_field_layout(ctx, |layout| {
+            max_size = cmp::max(max_size, layout.size);
+            max_align = cmp::max(max_align, layout.align);
+        });
 
         Some(Layout::new(max_size, max_align))
     }
@@ -1132,12 +1135,49 @@ impl CompInfo {
             CompFields::AfterComputingBitfieldUnits { ref fields, .. } => {
                 fields
             }
-            CompFields::BeforeComputingBitfieldUnits(_) => {
+            CompFields::BeforeComputingBitfieldUnits(..) => {
                 panic!("Should always have computed bitfield units first");
             }
         }
     }
 
+    fn has_fields(&self) -> bool {
+        match self.fields {
+            CompFields::ErrorComputingBitfieldUnits => false,
+            CompFields::AfterComputingBitfieldUnits { ref fields, .. } => {
+                !fields.is_empty()
+            }
+            CompFields::BeforeComputingBitfieldUnits(ref raw_fields) => {
+                !raw_fields.is_empty()
+            }
+        }
+    }
+
+    fn each_known_field_layout(
+        &self,
+        ctx: &BindgenContext,
+        mut callback: impl FnMut(Layout),
+    ) {
+        match self.fields {
+            CompFields::ErrorComputingBitfieldUnits => return,
+            CompFields::AfterComputingBitfieldUnits { ref fields, .. } => {
+                for field in fields.iter() {
+                    if let Some(layout) = field.layout(ctx) {
+                        callback(layout);
+                    }
+                }
+            }
+            CompFields::BeforeComputingBitfieldUnits(ref raw_fields) => {
+                for field in raw_fields.iter() {
+                    let field_ty = ctx.resolve_type(field.0.ty);
+                    if let Some(layout) = field_ty.layout(ctx) {
+                        callback(layout);
+                    }
+                }
+            }
+        }
+    }
+
     fn has_bitfields(&self) -> bool {
         match self.fields {
             CompFields::ErrorComputingBitfieldUnits => false,
@@ -1242,6 +1282,7 @@ impl CompInfo {
         let mut ci = CompInfo::new(kind);
         ci.is_forward_declaration =
             location.map_or(true, |cur| match cur.kind() {
+                CXCursor_ParmDecl => true,
                 CXCursor_StructDecl | CXCursor_UnionDecl |
                 CXCursor_ClassDecl => !cur.is_definition(),
                 _ => false,
@@ -1250,7 +1291,7 @@ impl CompInfo {
         let mut maybe_anonymous_struct_field = None;
         cursor.visit(|cur| {
             if cur.kind() != CXCursor_FieldDecl {
-                if let Some((ty, clang_ty, offset)) =
+                if let Some((ty, clang_ty, public, offset)) =
                     maybe_anonymous_struct_field.take()
                 {
                     if cur.kind() == CXCursor_TypedefDecl &&
@@ -1263,7 +1304,7 @@ impl CompInfo {
                         // nothing.
                     } else {
                         let field = RawField::new(
-                            None, ty, None, None, None, false, offset,
+                            None, ty, None, None, None, public, offset,
                         );
                         ci.fields.append_raw_field(field);
                     }
@@ -1272,7 +1313,7 @@ impl CompInfo {
 
             match cur.kind() {
                 CXCursor_FieldDecl => {
-                    if let Some((ty, clang_ty, offset)) =
+                    if let Some((ty, clang_ty, public, offset)) =
                         maybe_anonymous_struct_field.take()
                     {
                         let mut used = false;
@@ -1282,9 +1323,10 @@ impl CompInfo {
                             }
                             CXChildVisit_Continue
                         });
+
                         if !used {
                             let field = RawField::new(
-                                None, ty, None, None, None, false, offset,
+                                None, ty, None, None, None, public, offset,
                             );
                             ci.fields.append_raw_field(field);
                         }
@@ -1301,7 +1343,7 @@ impl CompInfo {
                     let comment = cur.raw_comment();
                     let annotations = Annotations::new(&cur);
                     let name = cur.spelling();
-                    let is_mutable = cursor.is_mutable_field();
+                    let is_public = cur.public_accessible();
                     let offset = cur.offset_of_field().ok();
 
                     // Name can be empty if there are bitfields, for example,
@@ -1319,7 +1361,7 @@ impl CompInfo {
                         comment,
                         annotations,
                         bit_width,
-                        is_mutable,
+                        is_public,
                         offset,
                     );
                     ci.fields.append_raw_field(field);
@@ -1376,9 +1418,11 @@ impl CompInfo {
                         cur.kind() != CXCursor_EnumDecl
                     {
                         let ty = cur.cur_type();
+                        let public = cur.public_accessible();
                         let offset = cur.offset_of_field().ok();
+
                         maybe_anonymous_struct_field =
-                            Some((inner, ty, offset));
+                            Some((inner, ty, public, offset));
                     }
                 }
                 CXCursor_PackedAttr => {
@@ -1411,6 +1455,8 @@ impl CompInfo {
                         ty: type_id,
                         kind,
                         field_name,
+                        is_pub: cur.access_specifier() ==
+                            clang_sys::CX_CXXPublic,
                     });
                 }
                 CXCursor_Constructor | CXCursor_Destructor |
@@ -1526,9 +1572,9 @@ impl CompInfo {
             CXChildVisit_Continue
         });
 
-        if let Some((ty, _, offset)) = maybe_anonymous_struct_field {
+        if let Some((ty, _, public, offset)) = maybe_anonymous_struct_field {
             let field =
-                RawField::new(None, ty, None, None, None, false, offset);
+                RawField::new(None, ty, None, None, None, public, offset);
             ci.fields.append_raw_field(field);
         }
 
@@ -1576,7 +1622,7 @@ impl CompInfo {
     pub fn is_packed(
         &self,
         ctx: &BindgenContext,
-        layout: &Option<Layout>,
+        layout: Option<&Layout>,
     ) -> bool {
         if self.packed_attr {
             return true;
@@ -1584,24 +1630,18 @@ impl CompInfo {
 
         // Even though `libclang` doesn't expose `#pragma packed(...)`, we can
         // detect it through its effects.
-        if let Some(ref parent_layout) = *layout {
-            if self.fields().iter().any(|f| match *f {
-                Field::Bitfields(ref unit) => {
-                    unit.layout().align > parent_layout.align
-                }
-                Field::DataMember(ref data) => {
-                    let field_ty = ctx.resolve_type(data.ty());
-                    field_ty.layout(ctx).map_or(false, |field_ty_layout| {
-                        field_ty_layout.align > parent_layout.align
-                    })
-                }
-            }) {
+        if let Some(parent_layout) = layout {
+            let mut packed = false;
+            self.each_known_field_layout(ctx, |layout| {
+                packed = packed || layout.align > parent_layout.align;
+            });
+            if packed {
                 info!("Found a struct that was defined within `#pragma packed(...)`");
                 return true;
-            } else if self.has_own_virtual_method {
-                if parent_layout.align == 1 {
-                    return true;
-                }
+            }
+
+            if self.has_own_virtual_method && parent_layout.align == 1 {
+                return true;
             }
         }
 
@@ -1614,10 +1654,13 @@ impl CompInfo {
     }
 
     /// Compute this compound structure's bitfield allocation units.
-    pub fn compute_bitfield_units(&mut self, ctx: &BindgenContext) {
-        // TODO(emilio): If we could detect #pragma packed here we'd fix layout
-        // tests in divide-by-zero-in-struct-layout.rs
-        self.fields.compute_bitfield_units(ctx, self.packed_attr)
+    pub fn compute_bitfield_units(
+        &mut self,
+        ctx: &BindgenContext,
+        layout: Option<&Layout>,
+    ) {
+        let packed = self.is_packed(ctx, layout);
+        self.fields.compute_bitfield_units(ctx, packed)
     }
 
     /// Assign for each anonymous field a generated name.
@@ -1630,7 +1673,12 @@ impl CompInfo {
     /// Requirements:
     ///     1. Current RustTarget allows for `untagged_union`
     ///     2. Each field can derive `Copy`
-    pub fn can_be_rust_union(&self, ctx: &BindgenContext) -> bool {
+    ///     3. It's not zero-sized.
+    pub fn can_be_rust_union(
+        &self,
+        ctx: &BindgenContext,
+        layout: Option<&Layout>,
+    ) -> bool {
         if !ctx.options().rust_features().untagged_union {
             return false;
         }
@@ -1639,12 +1687,22 @@ impl CompInfo {
             return false;
         }
 
-        self.fields().iter().all(|f| match *f {
+        let all_can_copy = self.fields().iter().all(|f| match *f {
             Field::DataMember(ref field_data) => {
                 field_data.ty().can_derive_copy(ctx)
             }
             Field::Bitfields(_) => true,
-        })
+        });
+
+        if !all_can_copy {
+            return false;
+        }
+
+        if layout.map_or(false, |l| l.size == 0) {
+            return false;
+        }
+
+        true
     }
 }
 
@@ -1739,7 +1797,7 @@ impl IsOpaque for CompInfo {
             //
             // See https://github.com/rust-lang/rust-bindgen/issues/537 and
             // https://github.com/rust-lang/rust/issues/33158
-            if self.is_packed(ctx, layout) &&
+            if self.is_packed(ctx, layout.as_ref()) &&
                 layout.map_or(false, |l| l.align > 1)
             {
                 warn!("Found a type that is both packed and aligned to greater than \
